/*
 * libxlsxwriter
 *
 * Copyright 2014-2022, John McNamara, jmcnamara@cpan.org. See LICENSE.txt.
 */

/**
 * @file xlsxwriter.h
 *
 * xlsxwriter - A library for creating Excel XLSX files.
 *
 */
#ifndef __LXW_XLSXWRITER_H__
#define __LXW_XLSXWRITER_H__

#include "xlsxwriter/workbook.h"
#include "xlsxwriter/worksheet.h"
#include "xlsxwriter/format.h"
#include "xlsxwriter/utility.h"

#define LXW_VERSION "1.1.5"
#define LXW_VERSION_ID 115
#define LXW_SOVERSION "5"

#endif /* __LXW_XLSXWRITER_H__ */
