###############################################################################
#
# Tests for libxlsxwriter.
#
# Copyright 2014-2022, John McNamara, jmcnamara@cpan.org
#

import base_test_class

class TestCompareXLSXFiles(base_test_class.XLSXBaseTest):
    """
    Test file created with libxlsxwriter against a file created by Excel.

    """

    def test_chart_axis01(self):
        self.run_exe_test('test_chart_axis01')

    def test_chart_axis02(self):
        self.run_exe_test('test_chart_axis02')

    def test_chart_axis04(self):
        self.run_exe_test('test_chart_axis04')

    def test_chart_axis05(self):
        self.run_exe_test('test_chart_axis05')

    def test_chart_axis06(self):
        self.run_exe_test('test_chart_axis06')

    def test_chart_axis07(self):
        self.run_exe_test('test_chart_axis07')

    def test_chart_axis08(self):
        self.run_exe_test('test_chart_axis08')

    def test_chart_axis09(self):
        self.run_exe_test('test_chart_axis09')

    def test_chart_axis10(self):
        self.run_exe_test('test_chart_axis10')

    def test_chart_axis11(self):
        self.run_exe_test('test_chart_axis11')

    def test_chart_axis12(self):
        self.run_exe_test('test_chart_axis12')

    def test_chart_axis13(self):
        self.run_exe_test('test_chart_axis13')

    # Stock chart not supported yet.
    # def test_chart_axis14(self):
    #     self.run_exe_test('test_chart_axis14')

    def test_chart_axis15(self):
        self.run_exe_test('test_chart_axis15')

    def test_chart_axis17(self):
        self.run_exe_test('test_chart_axis17')

    def test_chart_axis18(self):
        self.run_exe_test('test_chart_axis18')

    def test_chart_axis19(self):
        self.run_exe_test('test_chart_axis19')

    def test_chart_axis20(self):
        self.run_exe_test('test_chart_axis20')

    def test_chart_axis21(self):
        self.run_exe_test('test_chart_axis21')

    def test_chart_axis22(self):
        self.run_exe_test('test_chart_axis22')

    def test_chart_axis23(self):
        self.run_exe_test('test_chart_axis23')

    def test_chart_axis24(self):
        self.run_exe_test('test_chart_axis24')

    def test_chart_axis25(self):
        self.run_exe_test('test_chart_axis25')

    def test_chart_axis26(self):
        self.run_exe_test('test_chart_axis26')

    def test_chart_axis27(self):
        self.run_exe_test('test_chart_axis27')

    def test_chart_axis28(self):
        self.run_exe_test('test_chart_axis28')

    def test_chart_axis29(self):
        self.run_exe_test('test_chart_axis29')

    def test_chart_axis30(self):
        self.run_exe_test('test_chart_axis30')

    def test_chart_axis31(self):
        self.run_exe_test('test_chart_axis31')

    def test_chart_axis32(self):
        self.run_exe_test('test_chart_axis32')

    def test_chart_axis33(self):
        self.run_exe_test('test_chart_axis33')

    def test_chart_axis34(self):
        self.run_exe_test('test_chart_axis34')

    def test_chart_axis35(self):
        self.run_exe_test('test_chart_axis35')

    def test_chart_axis36(self):
        self.run_exe_test('test_chart_axis36')

    def test_chart_axis37(self):
        self.run_exe_test('test_chart_axis37')

    def test_chart_axis38(self):
        self.run_exe_test('test_chart_axis38')

    def test_chart_axis39(self):
        self.run_exe_test('test_chart_axis39')

    def test_chart_axis40(self):
        self.run_exe_test('test_chart_axis40')

    def test_chart_axis41(self):
        self.run_exe_test('test_chart_axis41')

    def test_chart_axis42(self):
        self.run_exe_test('test_chart_axis42')

    def test_chart_axis43(self):
        self.run_exe_test('test_chart_axis43')

    def test_chart_axis44(self):
        self.run_exe_test('test_chart_axis44')

    def test_chart_axis45(self):
        self.run_exe_test('test_chart_axis45')

    def test_chart_axis46(self):
        self.run_exe_test('test_chart_axis46')

    def test_chart_axis47(self):
        self.run_exe_test('test_chart_axis47')

    def test_chart_axis48(self):
        self.run_exe_test('test_chart_axis48')
