###############################################################################
#
# Tests for libxlsxwriter.
#
# Copyright 2014-2022, John McNamara, jmcnamara@cpan.org
#

import base_test_class

class TestCompareXLSXFiles(base_test_class.XLSXBaseTest):
    """
    Test file created with libxlsxwriter against a file created by Excel.

    """

    def test_chart_data_labels01(self):
        self.run_exe_test('test_chart_data_labels01')

    def test_chart_data_labels02(self):
        self.run_exe_test('test_chart_data_labels02')

    def test_chart_data_labels03(self):
        self.run_exe_test('test_chart_data_labels03')

    def test_chart_data_labels04(self):
        self.run_exe_test('test_chart_data_labels04')

    def test_chart_data_labels05(self):
        self.run_exe_test('test_chart_data_labels05')

    def test_chart_data_labels06(self):
        self.run_exe_test('test_chart_data_labels06')

    def test_chart_data_labels07(self):
        self.run_exe_test('test_chart_data_labels07')

    def test_chart_data_labels08(self):
        self.run_exe_test('test_chart_data_labels08')

    def test_chart_data_labels09(self):
        self.run_exe_test('test_chart_data_labels09')

    def test_chart_data_labels10(self):
        self.run_exe_test('test_chart_data_labels10')

    def test_chart_data_labels11(self):
        self.run_exe_test('test_chart_data_labels11')

    def test_chart_data_labels12(self):
        self.run_exe_test('test_chart_data_labels12')

    def test_chart_data_labels13(self):
        self.run_exe_test('test_chart_data_labels13')

    def test_chart_data_labels14(self):
        self.run_exe_test('test_chart_data_labels14')

    def test_chart_data_labels15(self):
        self.run_exe_test('test_chart_data_labels15')

    def test_chart_data_labels16(self):
        self.run_exe_test('test_chart_data_labels16')

    # Stock chart.

    def test_chart_data_labels18(self):
        self.run_exe_test('test_chart_data_labels18')

    def test_chart_data_labels19(self):
        self.run_exe_test('test_chart_data_labels19')

    def test_chart_data_labels20(self):
        self.run_exe_test('test_chart_data_labels20')

    def test_chart_data_labels21(self):
        self.run_exe_test('test_chart_data_labels21')

    def test_chart_data_labels22(self):
        self.run_exe_test('test_chart_data_labels22')

    def test_chart_data_labels23(self):
        self.run_exe_test('test_chart_data_labels23')

    def test_chart_data_labels24(self):
        self.run_exe_test('test_chart_data_labels24')

    def test_chart_data_labels25(self):
        self.run_exe_test('test_chart_data_labels25')

    def test_chart_data_labels26(self):
        self.run_exe_test('test_chart_data_labels26')

    def test_chart_data_labels27(self):
        self.run_exe_test('test_chart_data_labels27')

    def test_chart_data_labels28(self):
        self.run_exe_test('test_chart_data_labels28')

    def test_chart_data_labels29(self):
        self.run_exe_test('test_chart_data_labels29')

    def test_chart_data_labels30(self):
        self.run_exe_test('test_chart_data_labels30')

    def test_chart_data_labels31(self):
        self.run_exe_test('test_chart_data_labels31')

    def test_chart_data_labels32(self):
        self.run_exe_test('test_chart_data_labels32')

    def test_chart_data_labels33(self):
        self.run_exe_test('test_chart_data_labels33')

    def test_chart_data_labels34(self):
        self.run_exe_test('test_chart_data_labels34')

    def test_chart_data_labels35(self):
        self.run_exe_test('test_chart_data_labels35')

    def test_chart_data_labels36(self):
        self.run_exe_test('test_chart_data_labels36')

    def test_chart_data_labels37(self):
        self.run_exe_test('test_chart_data_labels37')

    def test_chart_data_labels38(self):
        self.run_exe_test('test_chart_data_labels38')

    def test_chart_data_labels40(self):
        self.run_exe_test('test_chart_data_labels40')

    def test_chart_data_labels41(self):
        self.run_exe_test('test_chart_data_labels41')

    def test_chart_data_labels42(self):
        self.run_exe_test('test_chart_data_labels42')

    def test_chart_data_labels43(self):
        self.run_exe_test('test_chart_data_labels43')

    def test_chart_data_labels44(self):
        self.run_exe_test('test_chart_data_labels44')

    def test_chart_data_labels45(self):
        self.run_exe_test('test_chart_data_labels45')

    def test_chart_data_labels46(self):
        self.run_exe_test('test_chart_data_labels46')

    def test_chart_data_labels47(self):
        self.run_exe_test('test_chart_data_labels47')

    def test_chart_data_labels48(self):
        self.run_exe_test('test_chart_data_labels48')

    def test_chart_data_labels49(self):
        self.run_exe_test('test_chart_data_labels49')

    def test_chart_data_labels50(self):
        self.run_exe_test('test_chart_data_labels50')
