/*
 * Test runner for xmlwriter using ctest.
 *
 * Copyright 2014-2022 John McNamara, jmcnamara@cpan.org
 *
 */
#define CTEST_MAIN

#include "../ctest.h"

int main(int argc, const char *argv[])
{
    return ctest_main(argc, argv);
}

